@@ -8,6 +8,13 @@ from applescript import kMissingValue
 
 from .script_loader import run_script
 
+""" In Catalina / Photos 5+, UUIDs in AppleScript have suffix that doesn't 
+    appear in actual database value.  These need to be dropped to be compatible
+    with osxphotos """
+UUID_SUFFIX_PHOTO = "/L0/001"
+UUID_SUFFIX_ALBUM = "/L0/040"
+UUID_SUFFIX_FOLDER = "/L0/020"
+
 
 class AppleScriptError(Exception):
     def __init__(self, *message):
@@ -18,6 +25,7 @@ class PhotosLibrary:
     def __init__(self):
         """ create new PhotosLibrary object and launch Photos """
         run_script("_photoslibrary_waitforphotos", 300)
+        self._version = str(run_script("_photoslibrary_version"))
 
     def activate(self):
         """ activate Photos.app """
@@ -27,6 +35,20 @@ class PhotosLibrary:
         """ quit Photos.app """
         run_script("_photoslibrary_quit")
 
+    @property
+    def running(self):
+        """ True if Photos is running, otherwise False """
+        return run_script("_photoslibrary_isrunning")
+
+    def hide(self):
+        """ Tell Photos to hide it's window """
+        run_script("_photoslibrary_hide")
+
+    @property
+    def hidden(self):
+        """ True if Photos is hidden (or not running), False if Photos is visible """
+        return run_script("_photoslibrary_hidden")
+
     @property
     def name(self):
         """ name of Photos.app """
@@ -35,7 +57,7 @@ class PhotosLibrary:
     @property
     def version(self):
         """ version of Photos.app as str """
-        return str(run_script("_photoslibrary_version"))
+        return self._version
 
     @property
     def frontmost(self):
@@ -46,10 +68,7 @@ class PhotosLibrary:
     def selection(self):
         """ List of Photo objects for currently selected photos or [] if no selection """
         uuids = run_script("_photoslibrary_get_selection")
-        if isinstance(uuids, list):
-            return [Photo(uuid) for uuid in uuids]
-        else:
-            return []
+        return [Photo(uuid) for uuid in uuids]
 
     @property
     def favorites(self):
@@ -373,27 +392,36 @@ class PhotosLibrary:
 
 class Album:
     def __init__(self, uuid):
+        id_ = uuid
+        # check to see if we need to add UUID suffix
+        if float(PhotosLibrary().version) >= 5.0:
+            if len(uuid.split("/")) == 1:
+                # osxphotos style UUID without the suffix
+                id_ = f"{uuid}{UUID_SUFFIX_ALBUM}"
+            else:
+                uuid = uuid.split("/")[0]
+
         valuuidalbum = run_script("_album_exists", uuid)
         if valuuidalbum:
-            self.id = uuid
-            self._uuid = uuid.split("/")[0]
+            self.id = id_
+            self._uuid = uuid
         else:
             raise ValueError(f"Invalid album id: {uuid}")
 
     @property
     def uuid(self):
-        """ UUID of Album """
+        """ UUID of Album (read only)"""
         return self._uuid
 
     @property
     def name(self):
-        """ name of album """
+        """ name of album (read/write) """
         name = run_script("_album_name", self.id)
         return name if name != kMissingValue else None
 
     @name.setter
     def name(self, name):
-        """ set name of photo """
+        """ set name of album """
         name = "" if name is None else name
         return run_script("_album_set_name", self.id, name)
 
@@ -443,17 +471,18 @@ class Album:
         photo_ids = run_script("_album_photos", self.id)
         return [Photo(uuid) for uuid in photo_ids]
 
-    def __len__(self):
-        return run_script("_album_len", self.id)
-
     def add(self, photos):
         """add photos from the library to album
 
         Args:
             photos: list of Photo objects to add to album
+
+        Returns:
+            list of Photo objects for added photos
         """
         uuids = [p.id for p in photos]
-        return run_script("_album_add", self.id, uuids)
+        added_ids = run_script("_album_add", self.id, uuids)
+        return [Photo(uuid) for uuid in added_ids]
 
     def import_photos(self, photo_paths, skip_duplicate_check=False):
         """import photos
@@ -481,6 +510,9 @@ class Album:
 
         Returns:
             list of names of exported photos
+
+        Note: if exporting large photos that are not yet downloaded from iCloud you may need 
+              to increase value of timeout.
         """
         photos = self.photos
         return [
@@ -488,7 +520,6 @@ class Album:
             for p in photos
         ]
 
-    # TODO: new album created at top level -- need to create new album at some path as old album
     def remove_by_id(self, photo_ids):
         """Remove photos from album.
             Note: Photos does not provide a way to remove photos from an album via AppleScript.
@@ -535,6 +566,9 @@ class Album:
         photo_uuids = [photo.id for photo in photos]
         return self.remove_by_id(photo_uuids)
 
+    def __len__(self):
+        return run_script("_album_len", self.id)
+
 
 class Folder:
     def __init__(self, uuid):
